@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4.cc,v 1.30 2001/04/03 17:42:41 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4.cc,v 1.34 2001/06/05 23:09:48 bmah Exp $";
 //
-// $Id: PctestIpv4.cc,v 1.30 2001/04/03 17:42:41 bmah Exp $
+// $Id: PctestIpv4.cc,v 1.34 2001/06/05 23:09:48 bmah Exp $
 //
 // PctestIpv4.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -30,6 +30,8 @@ static char rcsid[] = "$Id: PctestIpv4.cc,v 1.30 2001/04/03 17:42:41 bmah Exp $"
 #include "pc.h"
 #include "PctestIpv4.h"
 
+extern unsigned int Mtu;
+
 //
 // PctestIpv4::SetOriginName
 //
@@ -180,10 +182,12 @@ int PctestIpv4::SetTargetName(char *t)
 #endif /* HAVE_HERROR */
 
 	memset((void *) &targetAddress, 0, sizeof(struct in_addr));
-	len = strlen(t);
-	targetName = (char *) calloc(len + 1, sizeof(char));
-	strncpy(targetName, t, len + 1);
+	targetName = strdup(t);
 
+	if (targetName == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for target hostname.\n");
+	    return -1;
+	}
 	return -1;
     }
 
@@ -205,13 +209,11 @@ int PctestIpv4::SetTargetName(char *t)
     memcpy(&targetSocketAddress.sin_addr, host->h_addr_list[0], host->h_length);
 
     // Make a copy of the canonical hostname
-    len = strlen(host->h_name);
-    targetName = (char *) calloc(len + 1, sizeof(char));
+    targetName = strdup(host->h_name);
     if (targetName == NULL) {
-	fprintf(stderr, "Couldn't allocate memory for hostname.\n");
+	fprintf(stderr, "Couldn't allocate memory for target hostname.\n");
 	return -1;
     }
-    strncpy(targetName, host->h_name, len + 1);
 
     return 0;
 
@@ -224,14 +226,13 @@ int PctestIpv4::SetTargetName(char *t)
 //
 // Output:  In return value, returns socket number.
 //
-// Get input socket of an appropriate type.  We'll make this virtual 
-// shortly.
+// Get input socket of an appropriate type.
 //
 int PctestIpv4::GetSocketIn() {
     struct protoent *icmpProto = getprotobyname("icmp"); 
 				// be really anal-retentive
     if (icmpProto == NULL) {
-	fprintf(stderr, "Couldn't find ICMP protocol, using 1\n");
+	fprintf(stderr, "Warning: Couldn't determine ICMP protocol number, using 1\n");
 	proto = 1;		// instance variable of PctestIpv4
     }
     else {
@@ -294,3 +295,97 @@ char *PctestIpv4::GetName(void *a)
 
 }
 
+//
+// PctestIpv4::GenerateAdvancePacket
+//
+// Generate an ICMP throwaway packet, with all headers, owned by
+// the caller.  This is an ICMP echo reply packet, crafted in
+// such a way that it will (should) travel all the way to its target,
+// but be dropped by the target without generating anything coming
+// back at us.  In other words, it just occupies bandwidth on links.
+//
+char *PctestIpv4::GenerateAdvancePacket(TestRecord &tr) {
+
+    // Parameters stored as globals
+    extern unsigned int Tos;
+
+    // If the requested sending size is too small or too large, 
+    // then return an error.  The caller should have figured out the 
+    // minimum sending size by calling Pctest::GetMinSize().
+//    if ((tr.size < GetMinSize()) || (tr.size > IP_MAXPACKET)) {
+//	fprintf(stderr, "Bad packet size\n");
+//	return NULL;
+//    }
+
+    // Make up a ICMP packet to send out.
+    struct ip ipHeader;
+    memset(&ipHeader, 0, sizeof(ipHeader));
+#ifdef __osf__
+    // Tru64 <netinet/ip.h> does not declare ip_hl if __STDC__ == 1
+    ipHeader.ip_vhl = (sizeof(ip) >> 2) | (4 << 4);
+#else    
+    ipHeader.ip_hl = sizeof(ip) >> 2;
+    ipHeader.ip_v = 4;
+#endif /* __osf__ */
+    ipHeader.ip_tos = Tos;
+#ifdef linux
+    ipHeader.ip_len = htons(tr.size);
+#else
+    ipHeader.ip_len = Mtu;
+#endif /* linux */
+    ipHeader.ip_id = htons(0);
+#ifdef linux
+    ipHeader.ip_off = htons(IP_DF);
+#else
+    ipHeader.ip_off = IP_DF;
+#endif /* linux */
+    ipHeader.ip_ttl = MAXTTL;
+    ipHeader.ip_p = IPPROTO_ICMP;
+    ipHeader.ip_sum = 0;
+    memcpy(&(ipHeader.ip_src), &(originAddress), sizeof(struct in_addr));
+    memcpy(&(ipHeader.ip_dst), &(targetSocketAddress.sin_addr), sizeof(struct in_addr));
+
+    // Make up ICMP header.
+    int icmpPayloadSize = Mtu - sizeof(ip) - ICMP_MINLEN;
+				// need to hardcode size of headers for an ICMP
+				// echo reply packet
+    struct icmp icmpHeader;
+
+    icmpHeader.icmp_type = ICMP_ECHOREPLY;
+    icmpHeader.icmp_code = 0;
+    icmpHeader.icmp_cksum = htons(0); // compute checksum
+    icmpHeader.icmp_id = htons(icmpId);
+    icmpHeader.icmp_seq = htons(icmpSequence++);
+
+    // ICMP payload
+    char *icmpPayload;
+    icmpPayload = GeneratePayload(icmpPayloadSize);
+    if (icmpPayload == NULL) {
+	fprintf(stderr, "Couldn't allocate space for payload\n");
+	return NULL;
+    }
+
+    // Build the packet now.
+    char *ipPacket;
+    int ipPacketSize;
+    ipPacketSize = sizeof(ip) + ICMP_MINLEN + icmpPayloadSize;
+    ipPacket = new char[ipPacketSize];
+    if (ipPacket == NULL) {
+	fprintf(stderr, "Couldn't allocate space for packet\n");
+	return NULL;
+    }
+    memcpy(ipPacket, &ipHeader, sizeof(ipHeader));
+    memcpy(ipPacket + sizeof(ipHeader), &icmpHeader, ICMP_MINLEN);
+    memcpy(ipPacket + sizeof(ipHeader) + ICMP_MINLEN,
+	   icmpPayload, icmpPayloadSize);
+
+    // Compute ICMP checksum.  This is much simpler than the TCP or
+    // UDP checksums, because there is no pseudo-header.
+    u_int checksum;
+    checksum = (u_short) InCksum((u_short *) (ipPacket + sizeof(ipHeader)),
+				 ICMP_MINLEN + icmpPayloadSize);
+    ((icmp *)(ipPacket + sizeof(ipHeader)))->icmp_cksum = checksum;
+
+    return ipPacket;
+
+}
