@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv4.h,v 1.18 2001/01/19 22:18:46 bmah Exp $
+// $Id: PctestIpv4.h,v 1.19 2001/05/03 17:04:09 bmah Exp $
 //
 // PctestIpv4.h
 // Bruce A. Mah <bmah@acm.org>
@@ -47,12 +47,20 @@ class PctestIpv4 : public Pctest {
 	socketOut = 0;
 	socketIn = 0;
 	destPort = 32768; 
+
+	icmpId = (u_short) getpid(); // cache PID for ICMP ID field
+	icmpSequence = 0;	// init sequence number
     };
+
     PctestIpv4(int p) { 
 	socketOut = 0;
 	socketIn = 0;
 	destPort = p; 
+
+	icmpId = (u_short) getpid(); // cache PID for ICMP ID field
+	icmpSequence = 0;	// init sequence number
     };
+
     virtual ~PctestIpv4() { 
 	if (socketOut > 0) {
 	    close(socketOut);
@@ -84,8 +92,10 @@ class PctestIpv4 : public Pctest {
     int proto;			// (hopefully) ICMP protocol number
     int destPort;		// destination port number
 
+    u_short icmpId;		// ICMP ID
+    u_short icmpSequence;	// ICMP sequence number
+
+    virtual char *GenerateAdvancePacket(TestRecord &tr);
 };
 
 #endif /* PCTESTIPV4_H */
-
-
