@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv4Icmp.h,v 1.7 2001/04/03 19:41:35 bmah Exp $
+// $Id: PctestIpv4Icmp.h,v 1.13 2001/05/10 20:56:12 bmah Exp $
 //
 // PctestIpv4Icmp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -43,12 +43,10 @@ class PctestIpv4Icmp : public PctestIpv4 {
 
   public:
 
-    PctestIpv4Icmp() { 
-	PctestIpv4Icmp(0);
-    };
+    PctestIpv4Icmp();
     PctestIpv4Icmp(int p) { 
-	icmpId = (u_short) getpid(); // cache PID for ICMP ID field
-	icmpSequence = 0;	// init sequence number
+	fprintf(stderr, "PctestIpv4Icmp can't take port numbers\n");
+	exit(1);
     };
     virtual ~PctestIpv4Icmp() { 
     };
@@ -60,9 +58,6 @@ class PctestIpv4Icmp : public PctestIpv4 {
    
   protected:
 
-    u_short icmpId;		// ICMP ID
-    u_short icmpSequence;	// ICMP sequence number
-
 };
 
 #endif /* PCTESTIPV4ICMP_H */
