@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.18 2001/02/16 19:35:29 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.21 2001/05/11 16:53:22 bmah Exp $";
 //
-// $Id: PctestIpv4Raw.cc,v 1.18 2001/02/16 19:35:29 bmah Exp $
+// $Id: PctestIpv4Raw.cc,v 1.21 2001/05/11 16:53:22 bmah Exp $
 //
 // PctestIpv4Raw.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -26,10 +26,42 @@ static char rcsid[] = "$Id: PctestIpv4Raw.cc,v 1.18 2001/02/16 19:35:29 bmah Exp
 #include <netinet/ip_icmp.h>
 #include <arpa/inet.h>
 
+#ifdef HAVE_PCAP
+#include <pcap.h>
+#endif /* HAVE_PCAP */
+
 #include "pc.h"
 #include "PctestIpv4Raw.h"
 #include "TestRecord.h"
 
+extern unsigned Mtu;
+
+//
+// PctestIpv4Raw::PctestIpv4Raw
+//
+PctestIpv4Raw::PctestIpv4Raw(int p) { 
+    extern bool PcapFlag;
+
+#ifdef HAVE_PCAP
+    if (PcapFlag) {
+	// Initialize packet filter
+	// 
+	// XXX Note that we will see both inbound and outbound packets
+	// with this filter rule.  We *could* write up a new rule 
+	// to add a src host predicate.  XXX
+	if (pcap_compile(pc, &fp, "ip proto \\icmp", 1, maskp) < 0) {
+	    fprintf(stderr, "pcap_compile failed\n");
+	    exit(1);
+	}
+    
+	if (pcap_setfilter(pc, &fp) < 0) {
+	    fprintf(stderr, "pcap_setfilter failed\n");
+	    exit(1);
+	}
+    }
+#endif /* HAVE_PCAP */
+}
+
 //
 // GetSocketOut
 //
@@ -59,6 +91,16 @@ int PctestIpv4Raw::GetSocketOut() {
 	return rc;
     }
 
+    // Bind remote side of socket 
+    // (note that we need to have had PctestIpv4::SetTargetName()
+    // called first!
+    rc = connect(socketOut, (struct sockaddr *) &targetSocketAddress,
+		 sizeof(struct sockaddr_in));
+    if (rc < 0) {
+	perror("connect");
+	return rc;
+    }
+
 #ifdef linux
     // Linux needs SO_BSDCOMPAT enabled on our UDP socket, to avoid
     // getting ICMP errors when we send packets out.
@@ -87,14 +129,14 @@ int PctestIpv4Raw::GetSocketOut() {
 //
 int PctestIpv4Raw::Test(TestRecord &tr)
 {
-
-    struct timeval tvBefore, tvAfter;
     struct timeval timeout;
     int rc;			// syscall return code
     fd_set readFds;		// reading file descriptors
     int done = 0;
+    int i;			// generic loop counter
 
     // Parameters stored as globals
+    extern bool PcapFlag;
     extern unsigned int Tos;
     extern int Timeout;
 
@@ -188,6 +230,19 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     }
     ((udphdr *)(ipPacket + sizeof(ipHeader)))->uh_sum = ~checksum;
 
+    // If we need to construct some advance packets, generate 
+    // an image of said packet.
+    char *advancePacket;
+    if (tr.burst - 1 == 0) {
+	advancePacket = NULL;
+    }
+    else {
+	advancePacket = GenerateAdvancePacket(tr);
+	if (advancePacket == NULL) {
+	    return -1;
+	}
+    }
+
     // Use malloc(3) to allocate (memory-aligned) space for the inbound
     // packet.
     char *icmpPacket;
@@ -197,21 +252,35 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	return -1;
     }
 
+#ifdef HAVE_PCAP
+    if (!PcapFlag) {
+#endif /* HAVE_PCAP */
     // Set timeout value and socket select parameters
     timeout.tv_sec = Timeout;
     timeout.tv_usec = 0;
     FD_ZERO(&readFds);
     FD_SET(socketIn, &readFds);
+#ifdef HAVE_PCAP
+    }
+#endif /* HAVE_PCAP */
 
     // Timestamp before 
     gettimeofday(&tvBefore, NULL);
 
+    // Send advance packets if necessary
+    for (i = 1; i < tr.burst; i++) {
+	rc = send(socketOut, advancePacket, Mtu, 0);
+	if (rc < 0) {
+	    perror("send");
+	    goto exittest;
+	}
+    }
+    tr.size += (tr.burst - 1) * Mtu;
+
     // Send UDP packet
-    rc = sendto(socketOut, ipPacket, ipPacketSize, 0,
-		(struct sockaddr *) &targetSocketAddress,
-		sizeof(struct sockaddr_in));
+    rc = send(socketOut, ipPacket, ipPacketSize, 0);
     if (rc < 0) {
-	perror("sendto");
+	perror("send");
 	goto exittest;
     }
 
@@ -219,6 +288,25 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     // So we might end up doing this select/read several times.
     do {
 
+#ifdef HAVE_PCAP
+	if (PcapFlag) {
+	    // Wait for a packet.  Only take in one packet at a time.
+	    rc = pcap_dispatch(pc, 1, callback, (u_char *) this);
+	    if (rc < 0) {
+		fprintf(stderr, "pcap_dispatch failed\n");
+		goto exittest;
+	    }
+	    
+	    // The callback will handle our after- timestamp if
+	    // we get a packet, but if the read timeout fired first,
+	    // we'll need to do a timestamp here to be sure that tvAfter
+	    // has a valid value in it.
+	    if (rc == 0) {
+		gettimeofday(&tvAfter, NULL);
+	    }
+	}
+	else {
+#endif /* HAVE_PCAP */
 	// Select and wait for an ICMP response or a timeout
 	rc = select(FD_SETSIZE, &readFds, NULL, NULL, &timeout);
 	if (rc < 0) {
@@ -228,6 +316,10 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 
 	// Timestamp after and update test record timestamp fields
 	gettimeofday(&tvAfter, NULL);
+#ifdef HAVE_PCAP
+	}
+#endif /* HAVE_PCAP */
+
 	tr.tv.tv_sec = tvAfter.tv_sec - tvBefore.tv_sec - syscallTime.tv_sec;
 	tr.tv.tv_usec = tvAfter.tv_usec - tvBefore.tv_usec - syscallTime.tv_usec;
 	while (tr.tv.tv_usec < 0) {
@@ -239,12 +331,22 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	if (rc == 1) {
 	    IF_DEBUG(2, fprintf(stderr, "Response packet received\n"));
 
+#ifdef HAVE_PCAP
+	    if (PcapFlag) {
+		memcpy(icmpPacket, packet, packetLength);
+		tr.replsize = packetLength;
+	    }
+	    else {
+#endif /* HAVE_PCAP */
 	    rc = read(socketIn, icmpPacket, IP_MAXPACKET);
 	    if (rc < 0) {
 		perror("read");
 		goto exittest;
 	    }
 	    tr.replsize = rc;
+#ifdef HAVE_PCAP
+	    }
+#endif /* HAVE_PCAP */
 
 	    // Now parse the packet, doing a little error checking along
 	    // the way.  By the end, we'll have ipHeader and icmpHeader
@@ -345,7 +447,7 @@ int PctestIpv4Raw::Test(TestRecord &tr)
 	    done = 1;
 
 	}
-	else {
+	else if (tr.tv.tv_sec >= Timeout) {
 	    IF_DEBUG(2, fprintf(stderr, "Timeout\n"));
 
 	    tr.icmpSourceAddress = new char[sizeof(in_addr)];
@@ -362,6 +464,9 @@ int PctestIpv4Raw::Test(TestRecord &tr)
     rc = 0;
 
   exittest:
+    if (advancePacket) {
+	delete [] advancePacket;
+    }
     delete [] udpPayload;
     delete [] ipPacket;
     free(icmpPacket);
