@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv4Raw.h,v 1.7 2001/01/02 18:46:04 bmah Exp $
+// $Id: PctestIpv4Raw.h,v 1.8 2001/05/11 16:53:22 bmah Exp $
 //
 // PctestIpv4Raw.h
 // Bruce A. Mah <bmah@acm.org>
@@ -45,8 +45,7 @@ class PctestIpv4Raw : public PctestIpv4Udp {
 
     PctestIpv4Raw() { 
     };
-    PctestIpv4Raw(int p) { 
-    };
+    PctestIpv4Raw(int p);
     virtual ~PctestIpv4Raw() { 
     };
 
