@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv4Tcp.h,v 1.4 2001/02/01 21:38:49 bmah Exp $
+// $Id: PctestIpv4Tcp.h,v 1.5 2001/05/10 20:56:12 bmah Exp $
 //
 // PctestIpv4Tcp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -45,8 +45,7 @@ class PctestIpv4Tcp : public PctestIpv4 {
 
     PctestIpv4Tcp() { 
     };
-    PctestIpv4Tcp(int p) { 
-    };
+    PctestIpv4Tcp(int p);
     virtual ~PctestIpv4Tcp() { 
     };
 
