@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv4Udp.cc,v 1.19 2001/02/16 19:35:29 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv4Udp.cc,v 1.21 2001/05/11 16:11:27 bmah Exp $";
 //
-// $Id: PctestIpv4Udp.cc,v 1.19 2001/02/16 19:35:29 bmah Exp $
+// $Id: PctestIpv4Udp.cc,v 1.21 2001/05/11 16:11:27 bmah Exp $
 //
 // PctestIpv4Udp.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -94,8 +94,6 @@ int PctestIpv4Udp::GetSocketOut() {
 //
 int PctestIpv4Udp::Test(TestRecord &tr)
 {
-
-    struct timeval tvBefore, tvAfter;
     struct timeval timeout;
     int rc;			// syscall return code
     fd_set readFds;		// reading file descriptors
@@ -112,6 +110,13 @@ int PctestIpv4Udp::Test(TestRecord &tr)
 	return -1;
     }
 
+    // This protocol module doesn't support advance packets
+    if (tr.burst > 1) {
+	fprintf(stderr, 
+		"Protocol module has no support for burst parameter > 1\n");
+	return -1;
+    }
+
     // Make up a UDP packet to send out.
     int udpPayloadSize = tr.size - sizeof(ip) - sizeof(udphdr);
     char *udpPayload;
