@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: PctestIpv6.cc,v 1.24 2001/02/01 21:38:52 bmah Exp $";
+static char rcsid[] = "$Id: PctestIpv6.cc,v 1.31 2001/06/05 23:09:48 bmah Exp $";
 //
-// $Id: PctestIpv6.cc,v 1.24 2001/02/01 21:38:52 bmah Exp $
+// $Id: PctestIpv6.cc,v 1.31 2001/06/05 23:09:48 bmah Exp $
 //
 // PctestIpv6.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -21,17 +21,29 @@ static char rcsid[] = "$Id: PctestIpv6.cc,v 1.24 2001/02/01 21:38:52 bmah Exp $"
 #include <sys/param.h>
 #include <netdb.h>
 #include <netinet/in.h>
+
+#ifdef NEED_NRL_IPV6_HACK
+#include <netinet6/in6.h>
+#endif /* NEED_NRL_IPV6_HACK */
+
 #include <netinet/in_systm.h>
+
+#ifdef NEED_NRL_IPV6_HACK
+#include <netinet6/ipv6.h>
+#include <netinet6/icmpv6.h>
+#else
 #include <netinet/ip6.h>
-#include <netinet/udp.h>
 #include <netinet/icmp6.h>
+#endif /* NEED_NRL_IPV6_HACK */
+
+#include <netinet/udp.h>
 #include <arpa/inet.h>
 
 #include "pc.h"
 #include "PctestIpv6.h"
 
 //
-// PctestIpv4::SetOriginName
+// PctestIpv6::SetOriginName
 //
 // Input:  origin hostname
 //
@@ -183,10 +195,12 @@ int PctestIpv6::SetTargetName(char *t)
 	fprintf(stderr, "%s: %s\n", t, gai_strerror(ecode));
 
 	memset((void *) &targetAddress, 0, sizeof(struct in6_addr));
-	len = strlen(t);
-	targetName = (char *) calloc(len + 1, sizeof(char));
-	strncpy(targetName, t, len + 1);
+	targetName = strdup(t);
 
+	if (targetName == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for target hostname.\n");
+	    return -1;
+	}
 	return -1;
     }
 
@@ -225,20 +239,21 @@ int PctestIpv6::SetTargetName(char *t)
 	   &((struct sockaddr_in6 *) host->ai_addr)->sin6_addr,
            sizeof(struct in6_addr));
 
-    // Make a copy of the canonical hostname
-    len = strlen(host->ai_canonname);
-
-    targetName = (char *) calloc(len + 1, sizeof(char));
-    if (targetName == NULL) {
-	fprintf(stderr, "Couldn't allocate memory for hostname.\n");
-	return -1;
+    // Make a copy of the canonical hostname, if it exists.
+    if (host->ai_canonname) {
+	targetName = strdup(host->ai_canonname);
+    }
+    else {
+	fprintf(stderr, "Warning:  couldn't get canonical hostname.\n");
+	targetName = strdup(t);
     }
-
-    strncpy(targetName, host->ai_canonname, len + 1);
     freeaddrinfo(host);
 
+    if (targetName == NULL) {
+	fprintf(stderr, "Couldn't allocate memory for target hostname.\n");
+	return -1;
+    }
     return 0;
-
 }
 
 //
@@ -258,7 +273,7 @@ int PctestIpv6::GetSocketIn() {
 
     icmpProto = getprotobyname("ipv6-icmp"); // be really anal-retentive
     if (icmpProto == NULL) {
-	fprintf(stderr, "Couldn't find ICMP protocol, using 58\n");
+	fprintf(stderr, "Warning: Couldn't determine ICMPv6 protocol number, using 58\n");
 	proto = 58;		// instance variable of PctestIpv6
     }
     else {
@@ -361,4 +376,122 @@ char *PctestIpv6::GetName(void *a)
 
 }
 
+//
+// PctestIpv6::GetAdvanceSocketOut
+//
+// Input:  None
+//
+// Output:  In return value, returns socket number.
+//
+// Get output socket suitable for sending advance packets.
+//
+// NOTE:  Advance sockets (and the implementation of -b) for IPv6 is
+// slightly different between IPv6 and IPv4.  For IPv4, we use a
+// single raw socket (the same one that is used for the "real" probe
+// packets).  This works because we can (need to) construct the
+// entire IP packet, starting from the IP header.
+//
+// With IPv6, we don't control the IP header directly; raw sockets
+// do not allow us to do this.  So each socket is associated with
+// exactly one transport protocol (i.e. ICMPv6, UDP).  In addition, 
+// the ICMPv6 packets are treated specially by the stack, because
+// they have their pseudo-header calculated for them.  For these
+// reasons, we use a separate socket to send the advance packets
+// for IPv6.  This allows us to support -b independent of the actual
+// probe packet type.  (The same is not true for IPv4, where -b is
+// supported for raw UDP probes but not for regular SOCK_DGRAM-type
+// UDP probes).
+//
+int PctestIpv6::GetAdvanceSocketOut() {
+
+    int rc;
+    
+    advanceSocketOut = socket(AF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
+    if (advanceSocketOut < 0) {
+	perror("socket");
+	return advanceSocketOut;
+    }
+    
+    // Make up a socket address structure for the source address
+    // and attempt to bind the output socket to it.
+    struct sockaddr_in6 originSocketAddress;
+    memset((void *) &originSocketAddress, 0, sizeof(struct sockaddr_in6));
+    // See comments in PctestIpv6::SetTargetName() about the overloading
+    // of HAVE_SOCKADDR_SA_LEN.
+#ifdef HAVE_SOCKADDR_SA_LEN
+    originSocketAddress.sin6_len = sizeof(struct sockaddr_in6);
+#endif /* HAVE_SOCKADDR_SA_LEN */
+    originSocketAddress.sin6_family = AF_INET6;
+    originSocketAddress.sin6_port = htons(0);
+    memcpy(&(originSocketAddress.sin6_addr), &originAddress, sizeof(in6_addr));
+
+    rc = bind(advanceSocketOut, (struct sockaddr *) &originSocketAddress, 
+	      sizeof(originSocketAddress));
+    if (rc < 0) { 
+	perror("bind()");
+	return rc;
+    }
+
+    rc = connect(advanceSocketOut, (struct sockaddr *) &targetSocketAddress,
+		 sizeof(targetSocketAddress));
+    if (rc < 0) {
+	perror("connect");
+	return rc;
+    }
+
+    // Set TTL.
+    unsigned int hops = IPV6_MAXHLIM;
+    rc = setsockopt(advanceSocketOut, IPPROTO_IPV6, IPV6_UNICAST_HOPS, (char *) &hops, sizeof(hops));
+    if (rc < 0) {
+	perror("setsockopt(IPV6_UNICAST_HOPS)");
+	return rc;
+    }
+
+    return advanceSocketOut;
+}
+
+//
+// PctestIpv6::GenerateAdvancePacket
+//
+// Input:
+//
+// Output:
+//
+char *PctestIpv6::GenerateAdvancePacket(TestRecord &tr)
+{
+    extern unsigned int Mtu;
+
+    // Make up an ICMPv6 packet to send out.  We only need the ICMPv6
+    // header and payload, and the kernel will take care of computng
+    // the ICMPv6 checksum for us.
+    int icmp6PayloadSize = Mtu - sizeof(ip6_hdr) - sizeof(icmp6_hdr);
+    struct icmp6_hdr icmp6Header;
+
+    icmp6Header.icmp6_type = ICMP6_ECHO_REPLY;
+    icmp6Header.icmp6_code = 0;
+    icmp6Header.icmp6_id = htons(icmp6Id);
+    icmp6Header.icmp6_seq = htons(icmp6Sequence);
+
+    IF_DEBUG(2, fprintf(stdout, "test size %d, payload size %d\n", tr.size, icmp6PayloadSize));
+
+    // ICMPv6 payload
+    char *icmp6Payload;
+    icmp6Payload = GeneratePayload(icmp6PayloadSize);
+    if (icmp6Payload == NULL) {
+	fprintf(stderr, "Couldn't allocate space for payload\n");
+	return NULL;
+    }
+
+    // Build the packet (well, really just the ICMPv6 message)
+    // Once again, we don't need to bother with the ICMPv6 checksum;
+    // the kernel should deal with it for us.
+    char *icmp6Packet;
+    int icmp6PacketSize;
+    icmp6PacketSize = sizeof(icmp6_hdr) + icmp6PayloadSize;
+    icmp6Packet = new char[icmp6PacketSize];
+    memcpy(icmp6Packet, &icmp6Header, sizeof(icmp6Header));
+    memcpy(icmp6Packet + sizeof(icmp6Header), icmp6Payload, icmp6PayloadSize);
+
+    return icmp6Packet;
+}
 
