@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv6.h,v 1.9 2001/01/19 22:18:53 bmah Exp $
+// $Id: PctestIpv6.h,v 1.14 2001/06/05 17:17:03 bmah Exp $
 //
 // PctestIpv6.h
 // Bruce A. Mah <bmah@acm.org>
@@ -31,6 +31,10 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 
+#ifdef NEED_NRL_IPV6_HACK
+#include <netinet6/in6.h>
+#endif /* NEED_NRL_IPV6_HACK */
+
 #if HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
@@ -47,11 +51,17 @@ class PctestIpv6 : public Pctest {
 	socketOut = 0;
 	socketIn = 0;
 	destPort = 32768; 
+
+	icmp6Id = (u_short) getpid(); // cache PID for ICMP ID field
+	icmp6Sequence = 0;	// init sequence number
     };
     PctestIpv6(int p) { 
 	socketOut = 0;
 	socketIn = 0;
 	destPort = p; 
+
+	icmp6Id = (u_short) getpid(); // cache PID for ICMP ID field
+	icmp6Sequence = 0;	// init sequence number
     };
     virtual ~PctestIpv6() { 
 	if (socketOut > 0) {
@@ -60,6 +70,9 @@ class PctestIpv6 : public Pctest {
 	if (socketIn > 0) {
 	    close(socketIn);
 	}
+	if (advanceSocketOut > 0) {
+	    close(advanceSocketOut);
+	}
     };
 
     virtual int SetOriginName(char *origin);
@@ -82,9 +95,16 @@ class PctestIpv6 : public Pctest {
 
     int socketOut;		// output socket (RAW)
     int socketIn;		// input socket (ICMP)
+    int advanceSocketOut;	// output socket (RAW for ICMP)
     int proto;			// (hopefully) ICMP protocol number
     int destPort;		// destination port number
 
+    u_short icmp6Id;		// ICMP ID
+    u_short icmp6Sequence;	// ICMP sequence number
+
+    virtual int GetAdvanceSocketOut();
+    virtual char *GenerateAdvancePacket(TestRecord &tr);
+
 };
 
 #endif /* PCTESTIPV6_H */
