@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv6Icmp.h,v 1.2 2001/02/01 21:38:57 bmah Exp $
+// $Id: PctestIpv6Icmp.h,v 1.6 2001/06/05 17:17:03 bmah Exp $
 //
 // PctestIpv6Icmp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -32,6 +32,10 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 
+#ifdef NEED_NRL_IPV6_HACK
+#include <netinet6/in6.h>
+#endif /* NEED_NRL_IPV6_HACK */
+
 #if HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
@@ -43,12 +47,10 @@ class PctestIpv6Icmp : public PctestIpv6 {
 
   public:
 
-    PctestIpv6Icmp() { 
-	PctestIpv6Icmp(0);
-    };
+    PctestIpv6Icmp();
     PctestIpv6Icmp(int p) { 
-	icmp6Id = (u_short) getpid(); // cache PID for ICMP ID field
-	icmp6Sequence = 0;	// init sequence number
+	fprintf(stderr, "PctestIpv6Icmp can't take port numbers\n");
+	exit(1);
     };
     virtual ~PctestIpv6Icmp() { 
     };
@@ -59,9 +61,6 @@ class PctestIpv6Icmp : public PctestIpv6 {
     virtual PctestActionType GetAction(int icmp6_type, int icmp6_code);
    
   protected:
-    u_short icmp6Id;		// ICMP ID
-    u_short icmp6Sequence;	// ICMP sequence number
-
 };
 
 #endif /* PCTESTIPV6ICMP_H */
