@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: PctestIpv6Udp.h,v 1.6 2001/02/01 21:38:59 bmah Exp $
+// $Id: PctestIpv6Udp.h,v 1.7 2001/06/05 17:17:03 bmah Exp $
 //
 // PctestIpv6Udp.h
 // Bruce A. Mah <bmah@acm.org>
@@ -32,6 +32,10 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 
+#ifdef NEED_NRL_IPV6_HACK
+#include <netinet6/in6.h>
+#endif /* NEED_NRL_IPV6_HACK */
+
 #if HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
