@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: ResultTable.cc,v 1.24 2001/02/28 23:02:48 bmah Exp $";
+static char rcsid[] = "$Id: ResultTable.cc,v 1.28 2001/06/06 13:42:56 bmah Exp $";
 //
-// $Id: ResultTable.cc,v 1.24 2001/02/28 23:02:48 bmah Exp $
+// $Id: ResultTable.cc,v 1.28 2001/06/06 13:42:56 bmah Exp $
 //
 // ResultTable.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -31,11 +31,11 @@ static char rcsid[] = "$Id: ResultTable.cc,v 1.24 2001/02/28 23:02:48 bmah Exp $
 //
 ResultTable::ResultTable(unsigned int inc, unsigned int m, unsigned int b, 
 			 unsigned int r) : 
-    increment(inc), mtu(m), repetitions(r), burst(b), columns((burst+1)*mtu)
-// XXX Note the initialization of the columns member; we want to be
+    increment(inc), mtu(m), repetitions(r), burst(b), columns((burst+1)*mtu+1)
+// Note the initialization of the columns member; we want to be
 // able to hold the largest possible response packet.  (burst+1)*MTU should
-// work unless the user sets MTU to something small.  There should
-// be a better way to handle this.
+// work unless the user sets MTU to something small (then add one because 
+// packet sizes begin with 1, not 0).
 {
     
     int i;
@@ -45,7 +45,17 @@ ResultTable::ResultTable(unsigned int inc, unsigned int m, unsigned int b,
     // data = new (double *) [columns];
     typedef double *DoublePtr;
     data = new (double *[columns]);
+    if (data == NULL) {
+	fprintf(stderr, "Couldn't allocate data array for a ResultTable\n");
+	exit(1);
+    }
+
     used = new int[columns];
+    if (used == NULL) {
+	fprintf(stderr, "Couldn't allocate used array for a ResultTable\n");
+	exit(1);
+    }
+
     for (i = 0; i < columns; i++) {
 	data[i] = NULL;
 	used[i] = 0;
@@ -93,11 +103,15 @@ int ResultTable::put(int size, double time) {
     // Is the offset within the proper range for the table?
     offset = size2column(size);
     if ((offset < 0) || (offset >= columns)) {
+	fprintf(stderr, "Size %d out of bounds [0,%d)\n", offset, columns);
 	return -1;
     }
     
     // Any room left for more results in this column?
     if (used[offset] == repetitions) {
+	fprintf(stderr, 
+		"Too many repetitions for this size (%d >= %d)\n", 
+		used[offset], repetitions);
 	return -1;
     }
 
@@ -105,12 +119,14 @@ int ResultTable::put(int size, double time) {
     if (data[offset] == NULL) {
 	data[offset] = new double[repetitions];
 	if (data[offset] == NULL) {
+	    fprintf(stderr, "Couldn't allocate memory for new column\n");
 	    return -1;
 	}
     }
 
     // Store data
-    data[offset][used[offset]++] = time;
+    data[offset][used[offset]] = time;
+    (used[offset])++;
     return 0;
 
     // Invalidate result caches
@@ -395,10 +411,26 @@ void ResultTable::tau(double &a, double &b, double &blower, double &bupper)
 	
 	double *slopes;
 	slopes = new double[maxSlopes];
+	if (slopes == NULL) {
+	    fprintf(stderr, 
+		    "Couldn't allocate slopes array for a ResultTable\n");
+	    exit(1);
+	}
 
 	double *xvalues, *yvalues;
 	xvalues = new double[maxValues];
+	if (xvalues == NULL) {
+	    fprintf(stderr, 
+		    "Couldn't allocate xvalues array for a ResultTable\n");
+	    exit(1);
+	}
+
 	yvalues = new double[maxValues];
+	if (yvalues == NULL) {
+	    fprintf(stderr, 
+		    "Couldn't allocate yvalues array for a ResultTable\n");
+	    exit(1);
+	}
 
 	// Compute all the slopes.  Basically, we try to treat the
 	// maxSlopes datapoints as being in a single, 1-D array,
@@ -541,7 +573,18 @@ void ResultTable::lms(double &a, double &b, double &r2)
 	double *residuals;
 	double *ys;
 	residuals = new double[maxValues];
+	if (residuals == NULL) {
+	    fprintf(stderr, 
+		    "Couldn't allocate residuals array for a ResultTable\n");
+	    exit(1);
+	}
+
 	ys = new double[maxValues];
+	if (ys == NULL) {
+	    fprintf(stderr, 
+		    "Couldn't allocate ys array for a ResultTable\n");
+	    exit(1);
+	}
 
         estimatorFound = false;
 
@@ -753,12 +796,29 @@ void ResultTable::lmsint(double &a, double &b, double &r2)
     const unsigned int codscale = 1000;	// sqrt of scaling factor for coefficient of determination
 
     ys = new unsigned int[columns];
+    if (ys == NULL) {
+	fprintf(stderr, 
+		"Couldn't allocate ys array for a ResultTable\n");
+	exit(1);
+    }
+
     partialmins = new unsigned int[columns];
+    if (partialmins == NULL) {
+	fprintf(stderr, 
+		"Couldn't allocate partialmins array for a ResultTable\n");
+	exit(1);
+    }
+
     for (i = 0; i < columns; i++) {
 	// Convert dataset to integers representing microseconds.
 	partialmins[i] = (unsigned int) (data[i][0] * 1000000.0);
     }
     residuals = new unsigned int[columns*columns];
+    if (residuals == NULL) {
+	fprintf(stderr, 
+		"Couldn't allocate residuals array for a ResultTable\n");
+	exit(1);
+    }
     
     // Following code comes from the IOS version of pchar, hence
     // the C-style comments.
