@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //
-// $Id: ResultTable.h,v 1.14 2001/01/02 18:46:17 bmah Exp $
+// $Id: ResultTable.h,v 1.16 2001/02/28 23:02:48 bmah Exp $
 //
 // ResultTable.h
 // Bruce A. Mah <bmah@acm.org>
@@ -25,9 +25,10 @@
 class ResultTable {
     
   public:
-    const int increment, mtu, repetitions;
-    const int columns;
-    ResultTable(int i, int m, int r);
+    const unsigned int increment, mtu, burst, repetitions;
+    const unsigned int columns;
+    ResultTable(unsigned int i, unsigned int m, unsigned int b, 
+		unsigned int r);
     virtual ~ResultTable();
     
     int put(int size, double time);
