@@ -1,6 +1,6 @@
-static char rcsid[] = "$Id: main.cc,v 1.96 2001/04/03 19:41:35 bmah Exp $";
+static char rcsid[] = "$Id: main.cc,v 1.106 2001/06/04 17:54:16 bmah Exp $";
 //
-// $Id: main.cc,v 1.96 2001/04/03 19:41:35 bmah Exp $
+// $Id: main.cc,v 1.106 2001/06/04 17:54:16 bmah Exp $
 //
 // main.cc
 // Bruce A. Mah <bmah@acm.org>
@@ -19,7 +19,7 @@ static char rcsid[] = "$Id: main.cc,v 1.96 2001/04/03 19:41:35 bmah Exp $";
 
 #include <stdio.h>
 
-#if STDC_HEADERS
+#ifdef STDC_HEADERS
 #include <stdlib.h>
 #else
 extern "C" {
@@ -28,7 +28,7 @@ extern "C" {
 }
 #endif /* STDC_HEADERS */
 
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
@@ -41,7 +41,7 @@ extern "C" {
 #include <netinet/ip.h>
 #include <netinet/ip_icmp.h>
 
-#if HAVE_STRINGS_H
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
 
@@ -54,6 +54,7 @@ extern "C" {
 #include "PctestIpv4File.h"
 #ifdef HAVE_IPV6
 #include "PctestIpv6Icmp.h"
+#include "PctestIpv6Tcp.h"
 #include "PctestIpv6Udp.h"
 #include "PctestIpv6File.h"
 #endif /* HAVE_IPV6 */
@@ -78,10 +79,12 @@ void PrintIfInfo(const GetIfInfo *);
 AnalysisType Analysis = AnalysisLeastSquares;
 unsigned int Burst = 1;
 bool ChangeFlag = false;
+bool PcapFlag = false;
 int DebugLevel = 0;
 double Gap = 0.25;
 GapType GapDist = GapFixed;
 unsigned int Hops = 30;
+char *Interface = NULL;
 unsigned int Increment = 32;
 char *OriginHost = NULL;
 ModeType Mode = ModePchar;
@@ -109,6 +112,9 @@ ResultTable **PartialResults;
 ResultTable **PartialMins;
 const int MaxAddressesPerHop = 10;
 const int MaxSocketAddressLength = 255;
+#ifdef HAVE_PCAP
+char PcapErrBuf[PCAP_ERRBUF_SIZE];
+#endif /* HAVE_PCAP */
 
 //
 // Version
@@ -121,11 +127,12 @@ const int MaxSocketAddressLength = 255;
 //
 void VersionInfo()
 {
-    extern char *Version, *Copyright, *Build;
+    extern char *Version, *Copyright, *Build, *DFlags;
 
     fprintf(stderr, "%s\n", Version);
     fprintf(stderr, "%s\n", Copyright);
     fprintf(stderr, "%s\n", Build);
+    fprintf(stderr, "Compilation flags: %s\n", DFlags);
 
 }
 
@@ -139,7 +146,15 @@ void VersionInfo()
 // Print out invocation information
 //
 void Usage(char *argv0) {
-    fprintf(stderr, "Usage: %s [-a analysis] [-c] [-d debuglevel] [-g gap] [-G gaptype] [-h] [-H hops] [-i increment] [-m mtu] [-n] [-p protocol] [-P port] [-q] [-R reps] [-s hop] "
+    fprintf(stderr, "Usage: %s [-a analysis] [-b burst] [-c] "
+#ifdef HAVE_PCAP
+"[-C] "
+#endif /* HAVE_PCAP */
+"[-d debuglevel] [-g gap] [-G gaptype] [-h] [-H hops] "
+#ifdef HAVE_PCAP
+"[-i interface] "
+#endif /* HAVE_PCAP */
+"[-I increment] [-m mtu] [-n] [-p protocol] [-P port] [-q] [-R reps] [-s hop] "
 #ifdef HAVE_SNMP
 "[-S] "
 #endif /* HAVE_SNMP */
@@ -149,7 +164,11 @@ void Usage(char *argv0) {
     fprintf(stderr, "\t\t\tkendall\tLinear fit using Kendall's test statistic\n");
     fprintf(stderr, "\t\t\tlms\tLeast median of squares linear fit\n");
     fprintf(stderr, "\t\t\tlmsint\tLeast median of squares linear fit (integer computations)\n");
+    fprintf(stderr, "\t-b\t\tBurst size (default = %d)\n", Burst);
     fprintf(stderr, "\t-c\t\tIgnore route changes\n");
+#ifdef HAVE_PCAP
+    fprintf(stderr, "\t-C\t\tUse pcap packet capture facilities\n");
+#endif /* HAVE_PCAP */
     fprintf(stderr, "\t-d debuglevel\tSet debugging output level\n");
     fprintf(stderr, "\t-g gap\t\tInter-test gap in seconds (default = %0.2f)\n", Gap);
     fprintf(stderr, "\t-G gaptype\tInter-test gap type (default is fixed)\n");
@@ -157,7 +176,10 @@ void Usage(char *argv0) {
     fprintf(stderr, "\t\t\texp\tExponentially distributed random\n");
     fprintf(stderr, "\t-H hops\t\tMaximum number of hops (default = %d)\n", Hops);
     fprintf(stderr, "\t-h\t\tPrint this help information\n");
-    fprintf(stderr, "\t-i increment\tPacket size increment (default = %d)\n", Increment);
+#ifdef HAVE_PCAP
+    fprintf(stderr, "\t-i interface\tpcap interface\n");
+#endif /* HAVE_PCAP */
+    fprintf(stderr, "\t-I increment\tPacket size increment (default = %d)\n", Increment);
     fprintf(stderr, "\t-l host\t\tSet origin address of probes (defaults to hostname)\n");
     fprintf(stderr, "\t-m mtu\t\tMaximum packet size to check (default = %d)\n", Mtu);
     fprintf(stderr, "\t-M mode\t\tOperational mode (defaults to pchar)\n");
@@ -167,10 +189,13 @@ void Usage(char *argv0) {
     fprintf(stderr, "\t-p protocol\tNetwork protocol (default is ipv4udp)\n");
     fprintf(stderr, "\t\t\tipv4udp\t\tUDP over IPv4\n");
     fprintf(stderr, "\t\t\tipv4raw\t\tUDP over IPv4 (raw sockets)\n");
+#ifdef HAVE_PCAP
     fprintf(stderr, "\t\t\tipv4tcp\t\tTCP over IPv4 (raw sockets)\n");
+#endif /* HAVE_PCAP */
     fprintf(stderr, "\t\t\tipv4icmp\tICMP over IPv4 (raw sockets)\n");
 #ifdef HAVE_IPV6    
     fprintf(stderr, "\t\t\tipv6icmp\tICMPv6 over IPv6 (raw sockets)\n");
+    fprintf(stderr, "\t\t\tipv6tcp\t\tTCP over IPv6\n");
     fprintf(stderr, "\t\t\tipv6udp\t\tUDP over IPv6\n");
 #endif /* HAVE_IPV6 */
     fprintf(stderr, "\t-P port\t\tStarting port number (default = %d)\n", Port);
@@ -289,6 +314,9 @@ char *GetPrintableNetworkProtocol(NetworkProtocolType np)
     case (NetworkProtocolIpv6Icmp): 
 	return("ICMPv6/IPv6");
 	break;
+    case (NetworkProtocolIpv6Tcp): 
+	return("TCP/IPv6");
+	break;
     case (NetworkProtocolIpv6Udp): 
 	return("UDP/IPv6");
 	break;
@@ -312,7 +340,7 @@ int main(int argc, char **argv)
     Pctest *pct = NULL;		// test structure
 
     // Parse command-line arguments using getopt
-    while ((c = getopt(argc, argv, "a:cd:g:G:hH:i:l:m:M:np:P:qR:r:s:St:T:vVw:")) != -1) {
+    while ((c = getopt(argc, argv, "a:b:cCd:g:G:hH:i:I:l:m:M:np:P:qR:r:s:St:T:vVw:")) != -1) {
 
 	// Check for the different command-line flags we accept
 	switch (c) {
@@ -339,12 +367,34 @@ int main(int argc, char **argv)
 	    break;
 	}
 
+	// b: burst size
+	case 'b': {
+	    Burst = atoi(optarg);
+	    if (Burst < 1) {
+		fprintf(stderr, "Warning: burst size %d too small; resetting to 1\n");
+		Burst = 1;
+	    }
+	    break;
+	}
+
 	// c: ignore route changes
 	case 'c': {
 	    ChangeFlag = true;
 	    break;
 	}
 
+	// C: Use pcap packet capture facilities
+	case 'C': {
+#ifdef HAVE_PCAP	    
+	    PcapFlag = true;
+	    break;
+#else
+	    fprintf(stderr, "pcap unavailable in this build");
+	    Usage(argv[0]);
+	    exit(1);
+#endif /* HAVE_PCAP */
+	}
+
 	// d: set debugging level
 	case 'd': {
 	    DebugLevel = atoi(optarg);
@@ -390,8 +440,23 @@ int main(int argc, char **argv)
 	    break;
 	}
 
-	// i: set packet size increment
 	case 'i': {
+#ifdef HAVE_PCAP
+	    Interface = strdup(optarg);
+	    if (Interface == NULL) {
+		fprintf(stderr, "Couldn't allocate space for interface name\n");
+		exit(1);
+	    }
+	    break;
+#else
+	    fprintf(stderr, "pcap unavailable in this build");
+	    Usage(argv[0]);
+	    exit(1);
+#endif /* HAVE_PCAP */
+	}
+
+	// I: set packet size increment
+	case 'I': {
 	    Increment = atoi(optarg);
 	    break;
 	}
@@ -446,9 +511,11 @@ int main(int argc, char **argv)
 	    else if (strcasecmp(optarg, "ipv4raw") == 0) {
 		NetworkProtocol = NetworkProtocolIpv4Raw;
 	    }
+#ifdef HAVE_PCAP
 	    else if (strcasecmp(optarg, "ipv4tcp") == 0) {
 		NetworkProtocol = NetworkProtocolIpv4Tcp;
 	    }
+#endif /* HAVE_PCAP */
 	    else if (strcasecmp(optarg, "ipv4icmp") == 0) {
 		NetworkProtocol = NetworkProtocolIpv4Icmp;
 	    }
@@ -456,6 +523,9 @@ int main(int argc, char **argv)
 	    else if (strcasecmp(optarg, "ipv6icmp") == 0) {
 		NetworkProtocol = NetworkProtocolIpv6Icmp;
 	    }
+	    else if (strcasecmp(optarg, "ipv6tcp") == 0) {
+		NetworkProtocol = NetworkProtocolIpv6Tcp;
+	    }
 	    else if (strcasecmp(optarg, "ipv6udp") == 0) {
 		NetworkProtocol = NetworkProtocolIpv6Udp;
 	    }
@@ -724,6 +794,9 @@ int main(int argc, char **argv)
 	case (NetworkProtocolIpv6Icmp):
 	    pct = new PctestIpv6Icmp();
 	    break;
+	case (NetworkProtocolIpv6Tcp):
+	    pct = new PctestIpv6Tcp(Port);
+	    break;
 	case (NetworkProtocolIpv6Udp):
 	    pct = new PctestIpv6Udp(Port);
 	    break;
@@ -794,7 +867,7 @@ int main(int argc, char **argv)
 //
 void DoPchar(Pctest *pct) 
 {
-    int i, j, k, l;		// universal loop counters
+    int i, j, k, l, m;		// universal loop counters
     FILE *df = NULL;		// output file 
 
     // Generate set of packet sizes to test.  We'll test packets
@@ -838,6 +911,12 @@ void DoPchar(Pctest *pct)
 	fprintf(stdout, "pchar to %s (%s) using %s\n", pct->GetTargetName(), 
 		pct->GetPrintableAddress(),
 		GetPrintableNetworkProtocol(NetworkProtocol));
+	if (PcapFlag) {
+	    fprintf(stdout, "Using pcap capture on %s\n", Interface);
+	}
+	else {
+	    fprintf(stdout, "Using raw socket input\n");
+	}
 	fprintf(stdout, "Packet size increments from %d to %d by %d\n", 
 		pct->GetMinSize(), Mtu, Increment);
 	fprintf(stdout, "%d test(s) per repetition\n", testsPerRep);
@@ -937,6 +1016,7 @@ void DoPchar(Pctest *pct)
 	fprintf(df, "src %s\n", pct->GetPrintableAddress(pct->GetOriginAddress()));
 	fprintf(df, "dest %s\n", pct->GetPrintableAddress());
 	fprintf(df, "hops %d\n", Hops);
+	fprintf(df, "burst %d\n", Burst);
 	fprintf(df, "minsize %d\n", pct->GetMinSize());
 	fprintf(df, "increment %d\n", Increment);
 	fprintf(df, "mtu %d\n", Mtu);
@@ -992,6 +1072,7 @@ void DoPchar(Pctest *pct)
 	// packet sizes.
 	for (j = 0; j < Repetitions; j++) {
 	    for (k = 0; k < testsPerRep; k++) {
+		for (m = 1; m < Burst + 1; m++) {
 
 		int rc;		// syscall return code
 		double rtt;
@@ -999,13 +1080,14 @@ void DoPchar(Pctest *pct)
 
 		// Let the user know what we're doing
 		if (VerboseFlag) {
-		    fprintf(stderr, "    hop %2d  rep %3d  size %5d\r", 
-			    i+StartHop, j, packetSize[k]);
+		    fprintf(stderr, 
+			    "    hop %2d  rep %3d  burst %2d  size %5d\r", 
+			    i+StartHop, j, m, packetSize[k]);
 		}
 
 		// Set up variables for a test
 		tr = new TestRecord;
-		tr->burst = Burst;
+		tr->burst = m;
 		tr->size = packetSize[k];
 		if (tr->size < pct->GetMinSize()) {
 		    // Skip this test if the packet size is too small
@@ -1100,7 +1182,7 @@ void DoPchar(Pctest *pct)
 		    (pcta == PctestActionValidLasthop)) {
 
 		    if ((PartialResults[i]->put(
-			(tr->burst * tr->size) + tr->replsize, rtt)) < 0) {
+			tr->size + tr->replsize, rtt)) < 0) {
 			fprintf(stderr, "Couldn't store result\n");
 			abort();
 		    }
@@ -1120,7 +1202,6 @@ void DoPchar(Pctest *pct)
 		    // packet processing threw away everything except
 		    // time exceeded and port-unreachable.
 		    fprintf(stderr, "Unexpected response to probe\n");
-		    abort();
 		}
 
 		// If ICMP type was time exceeded, we know this is *not*
@@ -1174,11 +1255,9 @@ void DoPchar(Pctest *pct)
 		    if (rc < 0) {
 			perror("select");
 		    }
-
 		}
-		
 		delete tr;
-
+		}
 	    }
 	}
 
@@ -1313,14 +1392,14 @@ endreps:
 	    }
 
 	    if (addressesSeen > 0) {
-		for (l = 0; l < addressesSeen; l++) {
+		for (m = 0; m < addressesSeen; m++) {
 		    fprintf(stdout, "%2d: %s ", i+StartHop, 
-			    pct->GetPrintableAddress(checkAddress[l]));
+			    pct->GetPrintableAddress(checkAddress[m]));
 		    if (NumericFlag) {
-			fprintf(stdout, "(%s)\n", pct->GetPrintableAddress(checkAddress[l]));
+			fprintf(stdout, "(%s)\n", pct->GetPrintableAddress(checkAddress[m]));
 		    }
 		    else {
-			fprintf(stdout, "(%s)\n", pct->GetName(checkAddress[l]));
+			fprintf(stdout, "(%s)\n", pct->GetName(checkAddress[m]));
 		    }
 		}
 	    }
@@ -1412,8 +1491,17 @@ endreps:
 	    fprintf(stdout, "    End of path not reached after %d hops\n", pathLength);
 	}
 
-	fprintf(stdout, "    Start time:        %s", ctime(&(timeFirst.tv_sec)));
-	fprintf(stdout, "    End time:          %s", ctime(&(timeLast.tv_sec)));
+	// We don't know for sure that a time_t is the same size as 
+	// one of the members of a struct timeval.  So to make ctime(3)
+	// happy, we'll grab the value we want out of the timeval and
+	// put it in a real time_t before calling.
+	time_t temptime;
+
+	temptime = timeFirst.tv_sec;
+	fprintf(stdout, "    Start time:        %s", ctime(&temptime));
+
+	temptime = timeLast.tv_sec;
+	fprintf(stdout, "    End time:          %s", ctime(&temptime));
     }
 
     //
