@@ -1,5 +1,5 @@
 // -*- c++ -*-
-// $Id: pc.h,v 1.28 2001/01/26 17:26:56 bmah Exp $
+// $Id: pc.h,v 1.32 2001/06/07 15:01:41 bmah Exp $
 //
 // pc.h
 // Bruce A. Mah <bmah@acm.org>
@@ -47,7 +47,8 @@ typedef enum {
     ,
     NetworkProtocolIpv6Icmp,
     NetworkProtocolIpv6Udp,
-    NetworkProtocolIpv6File
+    NetworkProtocolIpv6File,
+    NetworkProtocolIpv6Tcp
 #endif /* HAVE_IPV6 */
 } NetworkProtocolType;
 
@@ -118,6 +119,15 @@ typedef enum {
 #define IPV6_MAXPACKET		65535
 #endif /* IPV6_MAXPACKET */
 
+// Some systems might not have IPV6_MAXHLIM.  This is intended for Linux
+// systems (observed on RH 7.1), but might be applicable elsewhere.
+// NRL-derived IPv6 systems have this constant defined below.
+#ifndef IPV6_MAXHLIM
+#ifndef NEED_NRL_IPV6_HACK
+#define IPV6_MAXHLIM 255
+#endif /* NEED_NRL_IPV6_HACK */
+#endif /* IPV6_MAXHLIM */
+
 // Define ICMP unreachable codes that might not be otherwise
 // available.  (Solaris 2.5.1 and 2.6 have this problem.)
 #define ICMP_UNREACH_FILTER_PROHIB	13
@@ -137,5 +147,34 @@ extern "C" {
 }
 #endif /* NEED_RANDOM_PROTO */
 
+// NRL IPv6 stack hacks.  Basically, they define data structures,
+// constants, etc. with different names than KAME (and, apparently,
+// the API RFCs).  Some of these we can take care of with some
+// preprocessor definitions; others require some different headers.
+#ifdef NEED_NRL_IPV6_HACK
+#define ip6_hdr		ipv6hdr
+#define ip6_nxt		ipv6_nextheader
+
+#define IPV6_MAXHLIM	IPV6_HOPLIMIT
+
+#define icmp6_hdr	icmpv6hdr
+#define icmp6_type	icmpv6_type
+#define icmp6_code	icmpv6_code
+#define icmp6_id	icmpv6_id
+#define icmp6_seq	icmpv6_seq
+
+#define ICMP6_ECHO_REQUEST		ICMPV6_ECHO_REQUEST
+#define ICMP6_ECHO_REPLY		ICMPV6_ECHO_REPLY
+#define ICMP6_TIME_EXCEEDED		ICMPV6_TIME_EXCEEDED
+#define ICMP6_DST_UNREACH		ICMPV6_DST_UNREACH
+#define ICMP6_DST_UNREACH_ADMIN		ICMPV6_UNREACH_ADMIN
+#define ICMP6_DST_UNREACH_NOPORT	ICMPV6_UNREACH_PORT
+
+#define icmp6_filter			icmpv6_filter
+#define ICMP6_FILTER		 	ICMPV6_FILTER
+#define ICMP6_FILTER_SETBLOCKALL	ICMPV6_FILTER_SETBLOCKALL
+#define ICMP6_FILTER_SETPASS		ICMPV6_FILTER_SETPASS
+#endif /* NEED_NRL_IPV6_HACK */
+
 #endif /* PC_H */
 
